function [ fh ] = plotFeasibleSet2DseveralConstraints(problem,indicesFreeParameters,valuesFixedParameters,...
    lb,ub,nrPointsInDir,SemiConstraints,OtherConstraints,coloring)
%SemiConstraints is a vector containing all the semi-infinite constraints
%that should be considered. OtherConstraints is 0 or 1 and contains the
%information wether other Constraints should be considered or not.

%Global settings
NrSemiInfConstrs=problem.getNrSemiInfConstrs();
NrPointsInDir=nrPointsInDir;
if NrPointsInDir>100
    warning('''NrPointsInDir^2'' lower level problems have to be solved. This may take some time')
end
y1Range = ub(1)-lb(1);
y2Range = ub(2)-lb(2);

width = 14;
height = y2Range/y1Range*width;

fh = fig('width',14, 'height', height);
hold on

%Settings for the solver
myLLPfminconOptions = optimset('Algorithm', 'sqp');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'Display', 'off');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxFunEvals', Inf);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxIter', 1000);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FunValCheck', 'on');
%myLLPfminconOptions = optimset(myLLPfminconOptions, 'GradObj', 'on', 'GradConstr', 'on', 'DerivativeCheck', der_check);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'ScaleProblem', 'obj-and-constr');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FinDiffType', 'central');

%compute x data
x1_temp=linspace(lb(1),ub(1),NrPointsInDir);
x2_temp=linspace(lb(2),ub(2),NrPointsInDir);
[x1,x2]=meshgrid(x1_temp,x2_temp);

%initialize phi
phi=-inf*ones(NrPointsInDir);

%check if indices ar in the right order
if indicesFreeParameters(1)>indicesFreeParameters(2)
    temp=indicesFreeParameters(1);
    indicesFreeParameters(1)=indicesFreeParameters(2);
    indicesFreeParameters(2)=temp;
    temp=lb(:,1);
    lb(:,1)=lb(:,2);
    lb(:,2)=temp;
    temp=ub(:,1);
    ub(:,1)=ub(:,2);
    ub(:,2)=temp;  
end

%check wether there are equality constraints
[A,b]=problem.getLinEqsOnUppLevVars();
if size(A,1)>0||problem.getNrUppLevNonlinEqs>0
    warning('This plot does not work for equality constraints. We ignore the equality constraints');            
end
        
        
for j=1:NrPointsInDir
    for j2=1:NrPointsInDir
        %Compute x-Point
        x=[valuesFixedParameters(1:indicesFreeParameters(1)-1),x1(j,j2),...
            valuesFixedParameters(indicesFreeParameters(1):indicesFreeParameters(2)-2),x2(j,j2),...
            valuesFixedParameters(indicesFreeParameters(2)-1:end)];
        %optimization of every lower level problem
        
        try
            for i=SemiConstraints
                k=problem.getIndexSetOfSemiInfConstr(i);   
                Yint = problem.getInitValsLowLevVars(k,x);
                [lbl, ubl] = problem.getBoundsOnLowLevVars(k,x);

                [Aineq,bineq] = problem.getLinIneqsOnLowLevVars (k,x);
                % max-problem    
                [~,fval] = fmincon(@LLPObjFun,Yint,Aineq,bineq,[],[],lbl,ubl,@LLPConFuns,myLLPfminconOptions,problem,i,x,k);
                phi(j,j2)=max(phi(j,j2),-fval);               
            end
        catch me
            phi(j,j2)=inf;
        end
        % non semi infinite constraints

        %compute non semi-infinite constraints
        %collect bounds
        [problb,probub]=problem.getBoundsOnUppLevVars();
        phi(j,j2)=max(phi(j,j2),problb(1)-x1(j,j2)); 
        phi(j,j2)=max(phi(j,j2),x1(j,j2)-probub(1));
        phi(j,j2)=max(phi(j,j2),problb(2)-x2(j,j2)); 
        phi(j,j2)=max(phi(j,j2),x2(j,j2)-probub(2));
        
        if OtherConstraints
            %collect linear constraints
            [A,b]=problem.getLinIneqsOnUppLevVars();
            for i2=1:size(A,1)
                phi(j,j2)=max(phi(j,j2),x*A(i2,:)'-b(i2));
            end


            if problem.getNrUppLevNonlinIneqs>0
                phi(j,j2)=max([phi(j,j2);problem.evalUppLevNonlinIneqConstrs(x)]);
            end
        end
        %display every 10th evalutation of the function phi
        if mod((j-1)*NrPointsInDir+j2,10)==0
            fprintf('The first %i points are computed\n',(j-1)*NrPointsInDir+j2);
        end
    end
end

phiPosNeg=phi<=0;
o=ones(NrPointsInDir);
Z=zeros(NrPointsInDir);
Z(phiPosNeg)=1*o(phiPosNeg);
C=contourf(x1,x2,Z,[1,1]);
if strcmp(lastwarn,'Contour not rendered for constant ZData')
    fill([lb(1),lb(1),ub(1),ub(1)],[lb(2),ub(2),ub(2),lb(2)],[135/255, 206/255, 250/255])
end


%Color the lines if desired

if coloring
    %initialize the colors
    %nrColors=size(SemiConstraints,2);
    colors=[1,1,1;1,0,1;0,1,1;1,1,0];
    %colors=[ones(nrColors,1),zeros(nrColors,1),linspace(0,1,nrColors)'];
    %loop over the boundary points
    for i=3:size(C,2) 
        %compute to which constraint the boundary line belongs
        %Therfore we take the middle point and compute to which constraint
        %this point belongs
            %Get the complete x-values
            x1=[valuesFixedParameters(1:indicesFreeParameters(1)-1),C(1,i),...
            valuesFixedParameters(indicesFreeParameters(1):indicesFreeParameters(2)-2),C(2,i),...
            valuesFixedParameters(indicesFreeParameters(2)-1:end)];
            x2=[valuesFixedParameters(1:indicesFreeParameters(1)-1),C(1,i-1),...
            valuesFixedParameters(indicesFreeParameters(1):indicesFreeParameters(2)-2),C(2,i-1),...
            valuesFixedParameters(indicesFreeParameters(2)-1:end)];
            x_mid=(x1+x2)/2;
        %compute the semi-infinite constraints
        Phi=zeros(size(SemiConstraints,2),1);
        ID=1;
        try
            for j=SemiConstraints
                 Yint = problem.getInitValsLowLevVars(k,x_mid);
                [lbl, ubl] = problem.getBoundsOnLowLevVars(k,x_mid);

                [Aineq,bineq] = problem.getLinIneqsOnLowLevVars (k,x_mid);
                % max-problem    
                [~,fval] = fmincon(@LLPObjFun,Yint,Aineq,bineq,[],[],lbl,ubl,@LLPConFuns,myLLPfminconOptions,problem,j,x,k);
                Phi(ID)=abs(fval);
                ID=ID+1;
            end
        catch me
            Phi(ID)=inf;
        end
        Phi;
        %compute the index of the constraint
        [minValue,index]=min(Phi);
        index=index+1;
        if OtherConstraints
            %Compare the value to the non semi-infinite constraints
            %linear constraints:            
            [A,b]=problem.getLinIneqsOnUppLevVars();
            if minValue>min(abs(A*x'-b))
                minValue=min(abs(A*x'-b));
                index=1;
            end

            if problem.getNrUppLevNonlinIneqs>0
                if minValue>min(abs(problem.evalUppLevNonlinIneqConstrs(x)));
                    minValue=min(abs(problem.evalUppLevNonlinIneqConstrs(x)));
                    index=1;
                end                
            end    
        end
        
        %compare all constraints with the bounds. If the boundary is caused
        %by the bounds do not plot it in a specific color.
        if minValue> min(abs(lb-(C(:,i)'+C(:,i-1)')/2))            
        else
            if minValue>min(abs(ub-(C(:,i)'+C(:,i-1)')/2))     
            else
            %Plot the line in the specific color
            plot(C(1,i-1:i),C(2,i-1:i),'-','color',colors(index,:),'LineWidth',2);
            end
        end
        
    end
end





%Set the colormap to get the right fillings
cmap=[135/255, 206/255, 250/255];
set(fh,'colormap',cmap)

%settings for plot
grid on;

set(gca, 'DataAspectRatio', [1 1 1]);
set(gca, 'Units', 'centimeters');
xlim([lb(1), ub(1)]);
ylim([lb(2), ub(2)]);
title('')
hold off



%Functions for the optimization
    function [g, grad_g_y] = LLPObjFun(y,GSIP_problem2,i,xval,~)

        g_temp = GSIP_problem2.evalSemiInfConstrs(i,xval,y);
        g=-g_temp; %max-problem


        grad_g_y_temp = GSIP_problem2.DySemiInfConstrs(i,xval,y);
        grad_g_y=-grad_g_y_temp';
    end

    %constraints of the lower level problem
    function [u, h, grad_u, grad_h] = LLPConFuns(y,GSIP_problem2,~,xval,k2)

        h = [];
        grad_h = [];
        assert(size(y,1)==1);
        u = GSIP_problem2.evalLowLevNonlinConstrs(k2,xval,y);

        grad_u = GSIP_problem2.DyLowLevNonlinConstrs(k2,xval,y)';


    end


end