function [works] = checkOneFunctionDerivative(problem,StringDFun,transposed,StringFun,posDVariable,SizeOfOutPut,varargin)
%checkOneFunctionDerivative(problem,StringDFun,transposed,StringFun,
%posDVariable,SizeOfOutPut,input1,input2,...)   Checks the derivative of a 
%single function. Checks whether the output sizes are correct, whether the
%Derivative funciton is executable and wheter the derivative coincides with
%the finite-differences derivative.
FunInputs=varargin;
NumberOfTestEvaluations=10;
NumberOfOutputs=length(SizeOfOutPut);
DF1=cell(1,NumberOfOutputs);
DF2=cell(1,NumberOfOutputs);
works=true;
DFun=str2func(StringDFun);
Fun=str2func(StringFun);
for i=1:NumberOfTestEvaluations
    try
    for j=1:2:length(FunInputs)
        TestParams{(j-1)/2+1}=FunInputs{j}+FunInputs{j+1}*(rand(size(FunInputs{j}))*2-1);
    end
    %evaluate the user supplied function and the finite difference derivative 
    [DF1{:}]=DFun(problem,TestParams{:});
    [DF2{:}]=getDerivativeViaDifferenceQuotient(@(x) RedFun(x),TestParams{posDVariable},SizeOfOutPut);
    
    
    for j=1:NumberOfOutputs
        
        if transposed
            DF1{j}=DF1{j}';
        else
        end
        %check the size

        if length(size(DF1{j}))~=length(size(DF2{j}))
            size(DF1{j})
            size(DF2{j})
            works =0;
        else
            works=works && min(size(DF1{j})==size(DF2{j}));
        end
        if works
            %check the value


            difference=abs(DF1{j}-DF2{j})./(abs(DF1{j})+(abs(DF1{j})<=10^-3)).*(abs(DF1{j})>10^-3)+abs(DF1{j}-DF2{j}).*(abs(DF1{j})<=10^-3);

            [maxDifference,pos]=max(difference(:));

            if maxDifference>10^-3
                warning('DerivativeCheck failed:\nUser-supplied and central finite-difference derivatives do not match within 1e-03 relative tolerance.');
                
                fprintf('Function %s does return',StringDFun)
                DF1{j}
                disp('Finite Differences:')
                DF2{j}
            end
            
            
        else
            
            %size(DF1{j})
            %size(DF2{j})
            switch j
                 case 1
                     warning('The %dst output of the function %s does not have the correct size',j,StringDFun);
                 case 2
                     warning('The %dnd output of the function %s does not have the correct size',j,StringDFun);
                 case 3
                     warning('The %drd output of the function %s does not have the correct size',j,StringDFun);
                 otherwise
                     warning('The %dth output of the function %s does not have the correct size',j,StringDFun);
            end
        end
    end
      catch me
          warning('The function %s does not work',StringDFun);
          disp(me.message);
          works=0;
      end
    if ~works
        return
    end
end

    function [varargout]=RedFun(x)
        varargout=cell(1,NumberOfOutputs);
        [varargout{:}]=Fun(problem,TestParams{1:posDVariable-1},x,TestParams{posDVariable+1:end});
    end

end

