function [] = checkUcSet( UcSet,checkDeriv,checkSecDeriv )
%checkUcSet(UcSet,checkDeriv,checkSecDeriv) checks all functions of the
%uncertainty set UcSet. If a output has the wrong size or an inconsistent 
%value a warning is thrown. It may happen, that in a first run only a few
%warnings are thrown. If they are corrected more mistakes may be found. If
%checkDeriv is 1, derivatives are checked. if checkSecDeriv is 1, seccond
%derivatives are checked.

        bgetDim=checkOneFunction(UcSet,'getDim',{[1;1]});
        bgetNrNonlinDescrFuns= checkOneFunction(UcSet,'getNrNonlinDescrFuns',{[1;1]});
        bgetNrNonlinEqConstrsOnParams=checkOneFunction(UcSet,'getNrNonlinEqConstrsOnParams',{[1;1]});
        bgetNrNonlinIneqConstrsOnParams=checkOneFunction(UcSet,'getNrNonlinIneqConstrsOnParams',{[1;1]}); 
        
        bgetInitValsParams=0;
        bgetLinEqsOnParams=0;
        bgetLinIneqsOnParams=0;
        bgetBoundsOnParams=0;
        if bgetDim
            nrOfParams=UcSet.getDim+1;
            bgetInitValsParams=checkOneFunction(UcSet,'getInitValsParams',{[1;nrOfParams]});
            bgetLinEqsOnParams=checkOneFunction(UcSet,'getLinEqsOnParams',{[inf;nrOfParams],[inf,1]});
            if bgetLinEqsOnParams
                [C,~]=UcSet.getLinEqsOnParams;
                bgetLinEqsOnParams=checkOneFunction(UcSet,'getLinEqsOnParams',{[size(C,1),nrOfParams],[size(C,1),1]});
            end
            bgetLinIneqsOnParams=checkOneFunction(UcSet,'getLinIneqsOnParams',{[inf;nrOfParams],[inf,1]});
            if bgetLinIneqsOnParams
                [A,~]=UcSet.getLinIneqsOnParams;
                bgetLinIneqsOnParams=checkOneFunction(UcSet,'getLinEqsOnParams',{[size(A,1),nrOfParams],[size(A,1),1]});
            end
            bgetBoundsOnParams=checkOneFunction(UcSet,'getBoundsOnParams',{[1,nrOfParams-1],[1;nrOfParams-1]});
        end
    
        if bgetInitValsParams  && bgetDim 
            n=UcSet.getDim+1;
            x=UcSet.getInitValsParams;
            dim=UcSet.getDim;
            bgetInteriorPoint=checkOneFunction(UcSet,'getInteriorPoint',{[1;dim]},x);       
 
            
            bevalNonlinDescrFuns=0;
            
            
            
           
            bevalInvTrafoFun=0;
            bevalTrafoFun=0;
            if bgetInteriorPoint && bgetNrNonlinDescrFuns
                nrOfUcSetConstrs=UcSet.getNrNonlinDescrFuns;
                y=UcSet.getInteriorPoint(x);
                bevalNonlinDescrFuns=checkOneFunction(UcSet,'evalNonlinDescrFuns',{[1;nrOfUcSetConstrs]},x,y);
                
                if bevalNonlinDescrFuns && checkDeriv
                min(bevalNonlinDescrFuns,checkOneFunctionDerivative(UcSet,'DxNonlinDescrFuns',0,'evalNonlinDescrFuns',1,{[nrOfUcSetConstrs]},x,1,y,1));
                min(bevalNonlinDescrFuns,checkOneFunctionDerivative(UcSet,'DyNonlinDescrFuns',0,'evalNonlinDescrFuns',2,{[nrOfUcSetConstrs]},x,1,y,1));
                end
                if bevalNonlinDescrFuns && checkSecDeriv
                    min(bevalNonlinDescrFuns,checkOneFunctionSecDerivative(UcSet,'DxDxNonlinDescrFuns','evalNonlinDescrFuns',1,{[nrOfUcSetConstrs]},x,1,y,1));
                    min(bevalNonlinDescrFuns,checkOneFunctionSecDerivative(UcSet,'DyDyNonlinDescrFuns','evalNonlinDescrFuns',2,{[nrOfUcSetConstrs]},x,1,y,1));
                    min(bevalNonlinDescrFuns,checkOneFunctionMixedSecDerivative(UcSet,'DxDyNonlinDescrFuns','evalNonlinDescrFuns',2,1,{[nrOfUcSetConstrs]},x,1,y,1));
                end
                
                bevalInvTrafoFun=checkOneFunction(UcSet,'evalInvTrafoFun',{[1;inf]},x,y);
                if bevalInvTrafoFun
                    z=UcSet.evalInvTrafoFun(x,y);
                    bevalTrafoFun=checkOneFunction(UcSet,'evalTrafoFun',{[1;dim]},x,z);
                    %bDxTrafoFun=checkOneFunction(UcSet,'DxTrafoFun',[2;dim],x,z);
                     
                    if bevalTrafoFun && checkDeriv
                        bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionDerivative(UcSet,'DxTrafoFun',0,'evalTrafoFun',1,{[dim]},x,1,z,1));
                        bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionDerivative(UcSet,'DzTrafoFun',0,'evalTrafoFun',2,{[dim]},x,1,z,1));
                    end
                    if bevalTrafoFun && checkSecDeriv
                        bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionSecDerivative(UcSet,'DxDxTrafoFun','evalTrafoFun',1,{[dim]},x,1,z,1));
                        bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionSecDerivative(UcSet,'DzDzTrafoFun','evalTrafoFun',2,{[dim]},x,1,z,1));
                        bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionMixedSecDerivative(UcSet,'DxDzTrafoFun','evalTrafoFun',2,1,{[dim]},x,1,z,1));
                    end                   
                end



            end
            bgetLinDescrFuns=checkOneFunction(UcSet,'getLinDescrFuns',{[inf;dim],[inf,1]},x);
            if bgetLinDescrFuns
                [A]=UcSet.getLinDescrFuns(x);
                nrLinIneqs=size(A,1);
                bgetLinDescrFuns=checkOneFunction(UcSet,'getLinDescrFuns',{[nrLinIneqs,dim],[nrLinIneqs,1]},x); 
            end
            if bgetLinDescrFuns && checkDeriv
                 [A]=UcSet.getLinDescrFuns(x);
                 nrLinIneqs=size(A,1);
                 bgetLinDescrFuns=min(bgetLinDescrFuns,checkOneFunctionDerivative(UcSet,'DxLinDescrFuns',0,'getLinDescrFuns',1,{[nrLinIneqs,dim],[nrLinIneqs,1]},x,1));
            end
            if bgetLinDescrFuns && checkSecDeriv
                 [A]=UcSet.getLinDescrFuns(x);
                 nrLinIneqs=size(A,1);
                 bgetLinDescrFuns=min(bgetLinDescrFuns,checkOneFunctionSecDerivative(UcSet,'DxDxLinDescrFuns','getLinDescrFuns',1,{[nrLinIneqs,dim],[nrLinIneqs,1]},x,1));
            end

            bgetBoundingBox=checkOneFunction(UcSet,'getBoundingBox',{[1;dim],[1,dim]},x);
            if bgetBoundingBox && checkDeriv
                bgetBoundingBox=min(bgetBoundingBox,checkOneFunctionDerivative(UcSet,'DxBoundingBox',0,'getBoundingBox',1,{[dim],[dim]},x,1));
            end
            if bgetBoundingBox && checkSecDeriv
                bgetBoundingBox=min(bgetBoundingBox,checkOneFunctionSecDerivative(UcSet,'DxDxBoundingBox','getBoundingBox',1,{[dim],[dim]},x,1));
            end
                     
            if bgetNrNonlinEqConstrsOnParams && bgetNrNonlinIneqConstrsOnParams
                n1=UcSet.getNrNonlinEqConstrsOnParams;
                n2=UcSet.getNrNonlinIneqConstrsOnParams;
                %constraints on parameters
                
                bevalNonlinEqConstrsOnParams=checkOneFunction(UcSet,'evalNonlinEqConstrsOnParams',{[1;n1]},x);

                if bevalNonlinEqConstrsOnParams && checkDeriv
                    bevalNonlinEqConstrsOnParams=checkOneFunctionDerivative(UcSet,'DxNonlinEqConstrsOnParams',0,'evalNonlinEqConstrsOnParams',1,{n1},x,1);
                end
                if bevalNonlinEqConstrsOnParams &&checkSecDeriv
                    bevalNonlinEqConstrsOnParams=checkOneFunctionSecDerivative(UcSet,'DxDxNonlinEqConstrsOnParams','evalNonlinEqConstrsOnParams',1,{n1},x,1);
                end


                bevalNonlinIneqConstrsOnParams=checkOneFunction(UcSet,'evalNonlinIneqConstrsOnParams',{[1;n2]},x);
                if bevalNonlinIneqConstrsOnParams && checkDeriv
                    bevalNonlinIneqConstrsOnParams=checkOneFunctionDerivative(UcSet,'DxNonlinIneqConstrsOnParams',0,'evalNonlinIneqConstrsOnParams',1,{n2},x,1);
                end
                if bevalNonlinIneqConstrsOnParams &&checkSecDeriv
                    bevalNonlinIneqConstrsOnParams=checkOneFunctionSecDerivative(UcSet,'DxDxNonlinIneqConstrsOnParams','evalNonlinIneqConstrsOnParams',1,{n2},x,1);
                end
               
            end
            allcorrect=min([bgetDim,  bgetNrNonlinDescrFuns,...
                bgetNrNonlinEqConstrsOnParams, bgetNrNonlinIneqConstrsOnParams, bgetInitValsParams, bgetLinEqsOnParams,...
                bgetLinIneqsOnParams, bgetBoundsOnParams,  bgetInteriorPoint,...
                bevalNonlinDescrFuns, bevalInvTrafoFun, bevalTrafoFun, bgetLinDescrFuns, bgetBoundingBox,...
                bevalNonlinEqConstrsOnParams, bevalNonlinIneqConstrsOnParams]); 


            if allcorrect
                disp(['All methods passed the test']);
            end  
        end
          
        


end

