
classdef ContainerC1 < Container
    %C=ContainerC1() creates an instance of the container C given by:
    %
    %       C={ y in R^2 with:
    %               -y1^2+y2 <=0
    %               y1-y2 -1<=0
    %               0.25y1^2+y2^2-1 <=0
    %               -2y1-y2^2-1 <=0
    %               -(y1+0.5)^2-(y2+0.5)^2 +0.04<=0
    %               -(y1-0.5)^2-(y2+0.5)^2 +0.04<=0
    %               -abs(y1)-y2 -0.5 <=0}
    %
    %   Bounding box: [-1.5,1.5]x[-1,1]
       
    properties (GetAccess=protected)
        dim =2;
        nrOfConstFuns=7;
        BoundingBox=[-1,-1;1.5,1];
    end
    
           
    methods
        %container functions
        function g=evalDescrFuns(~,i,y) 
            
            switch i
                case 1
                    g=-y(:,1).^2+y(:,2);
                case 2
                    g=y(:,1)-y(:,2).^2-1;
                case 3
                    g=1/4*y(:,1).^2+y(:,2).^2-1;
                case 4
                    g=-2*y(:,1)-y(:,2).^2-1;
                case 5
                    g=-(y(:,1)+0.5).^2-(y(:,2)+0.5).^2+0.04;
                case 6
                    g=-(y(:,1)-0.5).^2-(y(:,2)+0.5).^2+0.04;
                case 7
                    g= -abs(y(:,1))-y(:,2)-0.5;
            end 
        end
        
        %derivative, g7 is not differentiable.
        function Dy_g=DyDescrFuns(~,i,y) 
            n=size(y,1);
            o=ones(n,1);
           
            switch i
                case 1
                    Dy_g=[-2*(y(:,1)),o];
                case 2
                    Dy_g=[o,-2*y(:,2)];
                case 3
                    Dy_g=[1/2*y(:,1),2*y(:,2)];
                case 4
                    Dy_g=[-2*o,-2*y(:,2)];
                case 5
                    Dy_g=[-2*(y(:,1)+0.5),-2*(y(:,2)+0.5)];
                case 6
                    Dy_g=[-2*(y(:,1)-0.5),-2*(y(:,2)+0.5)];
                case 7
                    Dy_g=[-sign(y(:,1)),-o];
            end
        end
    end
    
    methods
        function plotContainer(obj)
            %implicit plot
            g1= @(y1,y2) -y1.^2+y2;
                
            g2= @(y1,y2) y1-y2.^2-1;
               
            g3= @(y1,y2) 1/4*y1.^2+y2.^2-1;
               
            g4= @(y1,y2) -2*y1-y2.^2-1;
            
            g5= @(y1,y2) -(y1+0.5).^2-(y2+0.5).^2+0.04;
                
            g6= @(y1,y2) -(y1-0.5).^2-(y2+0.5).^2+0.04;
            
            g7= @(y1,y2)  -abs(y1)-y2-0.5;
            
            
            
            m= @(y1,y2) max([g1(y1,y2),g2(y1,y2),g3(y1,y2),g4(y1,y2),g5(y1,y2),g6(y1,y2),g7(y1,y2)],[],2);
            
            h=ezplot(m,[-1.0,1.5,-1,1]);
            colormap([0 0 1])
            hold on
            
            
            

        end
    end
    
end
