
classdef ContainerC7 < Container
    %C=ContainerC7() creates an instance of the container C given by:
    %
    %       C={ y in R^2 with:
    %               0.3sin(pi*y1) - y2 <= 0
    %               y1^2 + 0.3y3^2 - 1<= 0
    %
    %   Bounding box: [-1,1]x[-0.5,2]
    
    
    properties (GetAccess=protected)
        dim  =2 ;                              
        nrOfConstFuns=2;                       
        BoundingBox=[-1,-0.5;1,2]

    end
    

    
    methods 
        %container functions
        function g=evalDescrFuns(~,i,y)   
            

            switch i
                case 1
                    g=3/10*sin(pi*y(:,1))-y(:,2);
                case 2
                    g=y(:,1).^2+3/10*y(:,2).^2-1;
            end

        end
    %derivative
        function Dy_g=DyDescrFuns(~,i,y)  
            m=size(y,1);
            
            switch i
                case 1
                    Dy_g=[3/10*pi*cos(pi*y(:,1)),-ones(m,1)];
                case 2
                    Dy_g=[2*y(:,1),3/5*y(:,2)];                
            end
        end
    end
    
    methods
        function plotContainer(obj)
            X1 =linspace(-1,1,100);
            X2 =linspace(1,-1,100);
            Y = [sqrt(10/3-10/3*X1.^2),3/10*sin(pi*X2)];     
   
            ph = fill([X1,X2], Y, [135/255, 206/255, 250/255]); % sky blue
            set(ph, 'EdgeColor', 'blue');
            hold on;
        end
    end
    
    
end