

classdef Problem11< ImplicitGSIPProblem
    % p=Problem11 creates an instance of the GSIP problem:
    %
    %       min   x2
    %
    %       s.t.  -y^3+x2 <=0 for all y in Y(x)
    %             -1 <=x1 <= 1
    %             -1 <=x2 <= 1
    %             2x2-x1^2 <=-10^-6
    %
    %       where:
    %
    %       Y(x)={y in R with
    %              2x2-y^3-x1^2 <= 0
    %              -1 <= y <= 0}
   
    
    properties(GetAccess=protected)

        
        nrOfUppLevVars=2;                          
        boundsOnUppLevVars=[-1,-1;1,1];             
        nrOfSemiInfConstrs=1;                       
        initValsOfUppLevVars=[0,1];                
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[];             
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=1;              
        
        
        nrOfLowLevVars=1;                          
        nrOfLowLevNonlinConstrs=1;                        
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end

    
    methods 
        %objective function
        function f=evalObjFun(obj,x)                   
            f=x(2);
            
        end
        function grad_f=gradObjFun(obj,x)                    
            grad_f=[0;1];
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)         
                                                         
            g=-y.^3+x(2); 
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)      
            n=size(y,1);
            Dx_g=[zeros(n,1),ones(n,1)];
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            Dy_g=-3*y.^2;                
        end
               
        
        function k=evalUppLevNonlinIneqConstrs(obj,x)       
            k=[2*x(2)-x(1)^2+1.e-6];                
        end
        function Dx_k=DxUppLevNonlinIneqConstrs(obj,x)      
            Dx_k=[-2*x(1),2];                
        end

        
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)          
            u=[2*x(2)-y^3-x(1)^2]; 
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)             
            Dy_u=[-3*y^2];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)             
            Dx_u=[-2*x(1),2];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)          
            [lb,ub]=obj.getInterval(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,~)    
             lb=-1;
             ub=0;
        end
                        
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)            
        
        
        function [lb,ub] = getInterval(obj,x)           
            if 2*x(2)-x(1)<-1
                lb=-1;
                ub=0;
            else
                lb=nthroot((2*x(2)-x(1)),3);
                ub=0;
            end
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            if 2*x(2)-x(1)<-1
                Dx_lb=[0,0];
                Dx_ub=[0,0];
            else
                Dx_lb=1/3*1/nthroot((2*x(2)-x(1)),3)^2*[-1,2];
                Dx_ub=[0,0];
            end
        end

    end
    
end

