classdef Problem3<ImplicitGSIPProblem
    % p=Problem2() creates an instance of the GSIP problem:
    %
    %       min   (x1+0.5-1/(1)x2
    %
    %       s.t.  y-x1+x2<=0 for all y in Y(x) 
    %             -2x1-x2<=-1-10^-6
    %             -5 <= x1 <= 5
    %             -5 <= x2 <= 5
    %       where:
    %
    %       Y(x)={y in R with
    %               y   <= x1-2
    %               -2y <= x2+3
    %               -5 <= y <= 5}
    
    % One constraint was added, such that Y(x) doesn't become empty.
    properties(GetAccess=protected)

        nrOfUppLevVars=2;                          
        boundsOnUppLevVars=[-5,-5;5,5];             
        nrOfSemiInfConstrs=1;                       
        initValsOfUppLevVars=[1,1];                
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[-2,-1, -1-1.e-6]; 
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;              
        
        
        nrOfLowLevVars=1;                          
        nrOfLowLevNonlinConstrs=0;                        
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end

    methods
        %objective function
        function f=evalObjFun(~,x)                    
            f=(x(1)+0.5-1/(1+sqrt(5)))^2+(x(2)-2.5)^2;
        end
        function grad_f=gradObjFun(~,x)                    
            grad_f=[2*(x(1)+0.5-1/(1+sqrt(5)));2*(x(2)-2.5)];
        end
        
        %upper level
        function g=evalSemiInfConstrs(~,~,x,y)       
                                                        
            g=y-x(1)+x(2);            
        end
        function Dx_g=DxSemiInfConstrs(~,~,~,y)      
            m=size(y,1);
            o=ones(m,1);
            Dx_g=[-o,o];                
        end
        function Dy_g=DySemiInfConstrs(~,~,~,y)
            m=size(y,1);
            o=ones(m,1);
            Dy_g=o;                
        end
                         
        function y=getInitValsLowLevVars(obj,~,x)
            lb=-5;
            t1=5/16*x(1);
            t2=5/11*(x(1)+x(2)+3);
            if(t1<=5)
                if(t2<=t1)
                    ub=t2;
                else
                    ub=t1;
                end
            else
                if(t2<=5)
                    ub=t2;
                else
                    ub=5;
                end
            end
            y=0.5*(lb+ub);
                            
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)            
            Dx_lb=[0,0];
            t1=5/16*x(1);
            t2=5/11*(x(1)+x(2)+3);
            if(t1<=5)
                if(t2<=t1)
                    Dx_ub=[5/11,5/11];
                else
                    Dx_ub=[5/16,0];
                end
            else
                if(t2<=5)
                    Dx_ub=[5/11,5/11];
                else
                    Dx_ub=[0,0];
                end
            end
            Dx_y=IntervalTransformationDxFun(Dx_lb,Dx_ub,z);
            Dx_y=0.5*(Dx_lb+Dx_ub);
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,~)    
             lb=-5;
             ub=5;
        end
        
        function [lb,ub]=getLowLevBoundingBox(obj,i,x)
            [lb,ub]=obj.getInterval(x);            
        end
        function [lb,ub]=getBoxZ(obj,i)
            lb=0;
            ub=1;
        end
        
        %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,k,x)
            A=[1;-2];
            b=[x(1)-2;x(2)+3];
        end
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(ub-y)/(ub-lb);
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
        function detDz_y=detDzTrafoFun(obj,i,x,z)
            m=size(z,1);
            [lb,ub]=obj.getInterval(x);
            detDz_y=(lb-ub)*ones(m,1);
        end

    end
    
    methods (Access=protected)
        function [lb,ub] = getInterval(~,x) 
            lb=max(-5,(-x(2)-3)/2);
            ub=min(5,x(1)-2);
        end
        function [Dx_lb,Dx_ub]=DxInterval(~,x)

            t1=(-x(2)-3)/2;
            t2=x(1)-2;
            if(t1>=-5)
                Dx_lb=[0,-0.5];
            else
                Dx_lb=[0,0];
            end
            if (t2<=5)
                Dx_ub=[1,0];
            else
                Dx_ub=[0,0];

            end
        end
    end
        
    
end

