classdef Problem6< ImplicitGSIPProblem
    % p=Problem6() creates an instance of the GSIP problem:
    %
    %       min   x
    %
    %       s.t.  -1.75-x-y <=0 for all y in Y(x) 
    %             -1 <= x <= 1
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               -1-x^2 <= y <= 1+x^2}
    
    properties(GetAccess=protected)
        
        
        nrOfUppLevVars=1;                        
        boundsOnUppLevVars=[-1;1];                  
        nrOfSemiInfConstrs=1;                       
        initValsOfUppLevVars=0;                    
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[];                   
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;              
        
        
        nrOfLowLevVars=1;                          
        nrOfLowLevNonlinConstrs=0;
        
        con2LowLev=1;
        nrOfIndexset=1;        

        
    end
    
    methods
        %objective function
        function f=evalObjFun(~,x)                    
            f=x;
        end
        function grad_f=gradObjFun(~,~)                  
            grad_f=1;
        end
        
        %upper level
        function g=evalSemiInfConstrs(~,~,x,y)       
                                                        
            g=-7/4-x-y;            
        end
        function Dx_g=DxSemiInfConstrs(~,~,~,y)      
            m=size(y,1);
            Dx_g=-ones(m,1);                
        end
        function Dy_g=DySemiInfConstrs(~,~,~,y)
            m=size(y,1);
            Dy_g=-ones(m,1);                
        end
        
        %lower level
        
        function y=getInitValsLowLevVars(~,~,~)         
            y=0;                
        end
        function Dx_y=DxStartingPointLowLev(~,~,~)
            Dx_y=0;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,k,x)  
             lb=-1-x^2;
             ub=1+x^2;
        end
                             
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);            
            z=(y-ub)/(lb-ub); 
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)
        
        function [lb,ub]=getInterval(~,x)                
            lb=-1-x^2;
            ub=1+x^2;            
        end
        function [Dx_lb,Dx_ub]=DxInterval(~,x)
            Dx_lb=-2*x;
            Dx_ub=2*x;            
        end
       
    end

    

    
end

