
classdef Problem9< ImplicitGSIPProblem
    % p=Problem9(m) creates an instance of the GSIP problem:
    %
    %       min   sum_i=1^m(3(m-i+1)/m*xi^2-2xi)
    %
    %       s.t.  sum_i=1^m((xi-y)^2) -10m <=0 for all y in Y(x) 
    %
    %       where:
    %
    %       Y(x)={y in R with
    %              -1 <= y <= sum_i=1^m(xi^2)+1}
    
    %The lower level problem is non-convex
 
    
    properties(GetAccess=protected)
        

        
        nrOfUppLevVars;                         
        boundsOnUppLevVars;                      
        nrOfSemiInfConstrs=1;                    
        initValsOfUppLevVars;                   
        
        linEqsOnUppLevVars=[];                  
        linIneqsOnUppLevVars=[];                
        nrOfNonlinEqsUppLev=0;             
        nrOfNonlinIneqsUppLev=0;           
        
        
        nrOfLowLevVars=1;                       
        nrOfLowLevNonlinConstrs=0;                     
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    
    methods
        %constructor
        function p=Problem9(nrOfParameters)    
            p.nrOfUppLevVars=nrOfParameters;
            p.initValsOfUppLevVars=zeros(1,nrOfParameters);
            p.boundsOnUppLevVars=[-inf*ones(1,nrOfParameters);inf*ones(1,nrOfParameters)];
            
        end
    end
    
    methods
        %objective function
        function f=evalObjFun(obj,x)                
            m=obj.nrOfUppLevVars;
            elementsSum=3*(m-(1:m)+1)/m.*(x.^2)-2*x;
            f=sum(elementsSum);
        end
        function grad_f=gradObjFun(obj,x)                    
            n=obj.nrOfUppLevVars;
            grad_f=(6*(n-(1:n)+1)/n.*x-2)';
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,~,x,y)          
           
            m=size(y,1);
            o1=ones(m,1);
            o2=ones(obj.nrOfUppLevVars,1);
            g=(o1*x-y*o2').^2*o2-10*obj.nrOfUppLevVars;
            
            
        end
        function Dx_g=DxSemiInfConstrs(obj,~,x,y)      
            m=size(y,1);
            Dx_g=zeros(m,obj.nrOfUppLevVars);
            for i=1:m
                Dx_g(i,:)=(2*(x-y(i)));
            end
        end
        function Dy_g=DySemiInfConstrs(~,~,x,y)
            m=size(y,1);
            Dy_g=ones(m,1);
            for i=1:m
                Dy_g(i,:)=-2*sum(x-y(i,:));
            end
        end
        
        %lower level
        
        function y=getInitValsLowLevVars(~,~,~)           
            y=0;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,~)
            n=obj.nrOfUppLevVars;
            Dx_y=zeros(1,n);
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,k,x)    
             lb=-1;
             ub=sum(x.^2)+1;
        end
                        
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
                        [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)
       
        function [lb,ub] = getInterval(~,x)              
            lb=-1;
            ub=sum(x.^2)+1;
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            Dx_lb=zeros(1,obj.nrOfUppLevVars);
            Dx_ub=2*x;
        end

       
    end

    

    
end
